;;; This used to be in file "x-interface.lisp"

(defvar *Possible-Messages* '((:any-tyi read-any)
			      (:ANY-TYI-NO-HANG read-any-char-no-hang)
			      (:baseline window-font-ascent)
			      :BEEP
			      :BITBLT
			      (:BOTTOM-MARGIN-SIZE window-bottom-margin-size)
			      (:BURY deexpose-window)
			      (:CENTER-AROUND center-window-around)
			      :CHARACTER-HEIGHT
			      :CHARACTER-WIDTH
			      :CHARACTER-WIDTH-IN-FONT
			      (:char-width default-char-width)
			      :CLEAR-BETWEEN-CURSORPOSES
			      (:clear-history clear-history)
			      (:clear-rest-of-line clear-rest-of-line)
			      (:clear-rest-of-window clear-rest-of-window)
			      (:clear-window clear-window)
			      (:COLOR-STREAM-P window-color-p)
			      (:COMPUTE-MOTION window-compute-motion)
			      (:configuration configuration)
			      :CONVERT-TO-ABSOLUTE-COORDINATES
			      :CONVERT-TO-INSIDE-COORDINATES
			      :CONVERT-TO-RELATIVE-COORDINATES
			      :CURRENT-BASELINE
			      (:current-font window-font)
			      (:CURRENT-LINE-HEIGHT window-line-height)
			      (:CURRENT-STYLE window-character-style)
			      :CURSOR-X
			      :CURSOR-Y
			      (:deexpose deexpose-window)
			      :DEFAULT-STYLE
			      (:edges window-edges)
			      :ERASE-DISPLAYED-PRESENTATION
			      (:expose expose-window)
			      (:EXPOSE-NEAR expose-window-near)
			      (:FRESH-LINE fresh-line)
			      (:get-pane get-pane)
			      (:height window-height)
			      (:HOME-CURSOR home-cursor)
			      (:HOME-DOWN home-down)
			      (:increment-cursorpos increment-cursorpos)
			      (:INFERIORS window-inferiors)
			      (:inside-edges window-inside-edges)
			      (:INSIDE-HEIGHT window-inside-height)
			      (:inside-size window-inside-size)
			      (:INSIDE-WIDTH window-inside-width)
			      (:KILL window-kill)
			      (:LEFT-MARGIN-SIZE window-left-margin-size)
			      (:LINE-HEIGHT window-line-height)
			      :LINE-IN
			      :LINE-OUT
			      (:MARGIN-COMPONENTS presentation-window-margins)
			      :MARGINS
			      (:MAXIMUM-X-POSITION presentation-window-max-x-position)
			      (:MAXIMUM-Y-POSITION presentation-window-max-y-position)
			      :MERGED-CURRENT-STYLE
			      :MINIMUM-X-POSITION
			      :MINIMUM-Y-POSITION
			      (:more-p window-more-p)
			      (:name window-name)
			      (:OPERATION-HANDLED-P window-operation-handled-p)
			      (:POSITION window-position)
			      :PRINT-SELF
			      (:read-cursorpos read-cursorpos)
			      (:read-location read-location)
			      :REFRESH
			      :REVERSE-VIDEO-P
			      (:RIGHT-MARGIN-SIZE window-right-margin-size)
			      :SAVE-BITS
			      :SELECT
			      :SET-BORDERS
			      (:set-configuration set-configuration)
			      (:set-cursorpos set-cursorpos)
			      (:SET-EDGES set-window-edges)
			      (:SET-INSIDE-SIZE set-window-inside-size)
			      (:set-label set-window-label)
			      (:SET-MAXIMUM-X-POSITION (setf presentation-window-max-x-position))
			      (:SET-MAXIMUM-Y-POSITION (setf presentation-window-max-y-position))
			      :SET-MINIMUM-X-POSITION
			      :SET-MINIMUM-Y-POSITION
			      (:SET-MORE-P (setf window-more-p))
			      (:set-position set-window-position)
			      :SET-SAVE-BITS
			      (:SET-SIZE set-window-size)
			      :SET-SIZE-IN-CHARACTERS
			      (:SET-SUPERIOR (setf window-superior))
			      (:set-viewport-position set-viewport-position)
			      (:set-vsp set-window-line-spacing)
			      (:size window-size)
			      :SIZE-IN-CHARACTERS
			      :STRING-IN
			      (:STRING-LENGTH window-string-length)
			      :STRING-LINE-IN
			      (:STRING-OUT window-string-out)
			      (:superior window-superior)
			      (:TOP-MARGIN-SIZE window-top-margin-size)
			      (:TYI read-char)
			      (:TYI-NO-HANG read-char-no-hang)
			      (:TYIPEEK peek-char)
			      (:TYO (write-char arg window arg))
			      (:UNTYI unread-char)
			      (:VIEWPORT-POSITION viewport-position)
			      (:visible-cursorpos-limits visible-cursorpos-limits)
			      (:vsp window-line-spacing)
			      (:width window-width)
			      (:WHICH-OPERATIONS which-operations)
			      :X-OFFSET
			      :Y-OFFSET
			      (:x-scroll-position x-scroll-position)
			      (:x-scroll-to x-scroll-to)
			      (:y-scroll-position y-scroll-position)
			      (:y-scroll-to y-scroll-to)))

(defvar *Messages-Handled*
	(copy-list (mapcan #'(lambda (item) (and (consp item) (list (first item))))
			   *Possible-Messages*)))

(dolist (pair *Possible-Messages*)
  (when (consp pair)
    (setf (get (first pair) 'real-function) (second pair))))

(defmethod which-operations ((window window))
;  (declare (ignore window))
  *Messages-Handled*)

(defmethod window-operation-handled-p ((window window) operation)
;  (declare (ignore window))
  (not (null (member operation *Messages-Handled*))))

(defmacro send (object message &rest args)
  (setq message (eval message)) ;;; yuk!!!
  (let ((function (get message 'real-function)))
    (cond ((and (consp function)
		(eq 'setf (first function)))
	   `(setf (,(second function) ,object) . ,args))
	  ((consp function)
	   (let ((window-position (position 'WINDOW function))
		 (first-args NIL))
	     (dotimes (x (%1- window-position))
	       (push (pop args) first-args))
	     `(,(first function) ,@(nreverse first-args) ,object . ,args)))
	  (function
	   `(,function ,object . ,args))
	  (T
	   (format T "No Translation for message ~A." message)
	   `(format T "No Translation for message ~A." ,message)))))


